@@ -123,9 +123,13 @@ if ! $skip_gnulib; then
     gettext-runtime-misc
     ansi-c++-opt
     csharpcomp-script
+    d
+    dcomp-script
     java
     javacomp-script
     manywarnings
+    modula2
+    modula2comp-script
   '
   $GNULIB_TOOL --dir=gettext-runtime --lib=libgrt --source-base=gnulib-lib --m4-base=gnulib-m4 --no-libtool --local-dir=gnulib-local --local-symlink \
     --import $GNULIB_MODULES_RUNTIME_FOR_SRC $GNULIB_MODULES_RUNTIME_OTHER || exit $?
@@ -178,6 +182,7 @@ if ! $skip_gnulib; then
     c-strcase
     c-strcasestr
     c-strstr
+    carray-list
     clean-temp
     closedir
     closeout
@@ -201,7 +206,12 @@ if ! $skip_gnulib; then
     getaddrinfo
     getline
     getopt-gnu
+    getrusage
     gettext-h
+    gocomp-script
+    hash-map
+    hash-set
+    hashkey-string
     iconv
     javacomp
     javaexec
@@ -218,6 +228,7 @@ if ! $skip_gnulib; then
     memset
     minmax
     mkdir
+    next-prime
     noreturn
     obstack
     open
@@ -230,6 +241,7 @@ if ! $skip_gnulib; then
     readdir
     relocatable-prog
     relocatable-script
+    set
     setlocale
     sf-istream
     sh-filename
@@ -237,6 +249,7 @@ if ! $skip_gnulib; then
     sigpipe
     sigprocmask
     spawn-pipe
+    stat-time
     stdio-h
     stdlib-h
     stpcpy
@@ -269,7 +282,11 @@ if ! $skip_gnulib; then
     unistr/u8-mbtouc
     unistr/u8-mbtoucr
     unistr/u8-uctomb
+    unistr/u16-check
+    unistr/u16-to-u8
     unistr/u16-mbtouc
+    unistr/u32-check
+    unistr/u32-to-u8
     uniwidth/width
     unlocked-io
     unsetenv
@@ -280,13 +297,17 @@ if ! $skip_gnulib; then
     xalloc
     xconcat-filename
     xerror
+    xlist
     xmalloca
+    xmap
     xmemdup0
+    xset
     xsetenv
     xstrerror
     xstriconv
     xstriconveh
     xstring-buffer
+    xstring-buffer-reversed
     xstring-desc
     xvasprintf
   '
@@ -326,10 +347,14 @@ if ! $skip_gnulib; then
     ansi-c++-opt
     csharpcomp-script
     csharpexec-script
+    d
+    dcomp-script
     java
     javacomp-script
     javaexec-script
     manywarnings
+    modula2
+    modula2comp-script
     stdint-h
     test-xfail
   '
@@ -343,8 +368,11 @@ if ! $skip_gnulib; then
   GNULIB_MODULES_LIBGETTEXTLIB="$GNULIB_MODULES_TOOLS_FOR_SRC $GNULIB_MODULES_TOOLS_FOR_SRC_COMMON_DEPENDENCIES $GNULIB_MODULES_TOOLS_OTHER"
   $GNULIB_TOOL --dir=gettext-tools --lib=libgettextlib --source-base=gnulib-lib --m4-base=gnulib-m4 --tests-base=gnulib-tests --makefile-name=Makefile.gnulib --libtool --with-tests --local-dir=gnulib-local --local-symlink \
     --import \
+    --avoid=float-h-tests \
+    --avoid=hashcode-string1 \
     --avoid=fdutimensat-tests --avoid=futimens-tests --avoid=utime-tests --avoid=utimens-tests --avoid=utimensat-tests \
-    --avoid=array-list-tests --avoid=linked-list-tests --avoid=linkedhash-list-tests \
+    --avoid=array-list-tests --avoid=array-map-tests --avoid=array-oset-tests --avoid=carray-list-tests --avoid=linked-list-tests --avoid=linkedhash-list-tests \
+    --avoid=uninorm/decomposing-form-tests \
     `for m in $GNULIB_MODULES_TOOLS_LIBUNISTRING_TESTS; do echo --avoid=$m; done` \
     $GNULIB_MODULES_LIBGETTEXTLIB || exit $?
   $GNULIB_TOOL --copy-file m4/libtextstyle.m4 gettext-tools/gnulib-m4/libtextstyle.m4 || exit $?
@@ -353,7 +381,7 @@ if ! $skip_gnulib; then
     mbrlen
     regex
   '
-  $GNULIB_TOOL --dir=gettext-tools --macro-prefix=grgl --lib=libgrep --source-base=libgrep --m4-base=libgrep/gnulib-m4 --witness-c-macro=IN_GETTEXT_TOOLS_LIBGREP --makefile-name=Makefile.gnulib --local-dir=gnulib-local --local-symlink \
+  $GNULIB_TOOL --dir=gettext-tools --macro-prefix=grgl --source-base=libgrep/gnulib-lib --m4-base=libgrep/gnulib-m4 --makefile-name=Makefile.gnulib --witness-c-macro=IN_GETTEXT_TOOLS_LIBGREP --local-dir=gnulib-local --local-symlink \
     --import \
     `for m in $GNULIB_MODULES_TOOLS_FOR_SRC_COMMON_DEPENDENCIES; do \
        if test \`$GNULIB_TOOL --extract-applicability $m\` != all; then \
