@@ -1,5 +1,5 @@
 /* Tcl format strings.
-   Copyright (C) 2001-2004, 2006-2007, 2009, 2019-2020, 2023 Free Software Foundation, Inc.
+   Copyright (C) 2001-2025 Free Software Foundation, Inc.
    Written by Bruno Haible <haible@clisp.cons.org>, 2002.
 
    This program is free software: you can redistribute it and/or modify
@@ -79,12 +79,6 @@ struct spec
   struct numbered_arg *numbered;
 };
 
-/* Locale independent test for a decimal digit.
-   Argument can be  'char' or 'unsigned char'.  (Whereas the argument of
-   <ctype.h> isdigit must be an 'unsigned char'.)  */
-#undef isdigit
-#define isdigit(c) ((unsigned int) ((c) - '0') < 10)
-
 
 static int
 numbered_arg_compare (const void *p1, const void *p2)
@@ -130,7 +124,7 @@ format_parse (const char *format, bool translated, char *fdi,
             enum format_arg_type type;
 
             is_numbered_arg = false;
-            if (isdigit (*format))
+            if (c_isdigit (*format))
               {
                 const char *f = format;
                 unsigned int m = 0;
@@ -140,7 +134,7 @@ format_parse (const char *format, bool translated, char *fdi,
                     m = 10 * m + (*f - '0');
                     f++;
                   }
-                while (isdigit (*f));
+                while (c_isdigit (*f));
 
                 if (*f == '$')
                   {
@@ -198,9 +192,9 @@ format_parse (const char *format, bool translated, char *fdi,
 
                 number++;
               }
-            else if (isdigit (*format))
+            else if (c_isdigit (*format))
               {
-                do format++; while (isdigit (*format));
+                do format++; while (c_isdigit (*format));
               }
 
             /* Parse precision.  */
@@ -223,9 +217,9 @@ format_parse (const char *format, bool translated, char *fdi,
 
                     number++;
                   }
-                else if (isdigit (*format))
+                else if (c_isdigit (*format))
                   {
-                    do format++; while (isdigit (*format));
+                    do format++; while (c_isdigit (*format));
                   }
               }
 
