@@ -1,5 +1,5 @@
 /* Extracts strings from C source file to Uniforum style .po file.
-   Copyright (C) 1995-2024 Free Software Foundation, Inc.
+   Copyright (C) 1995-2025 Free Software Foundation, Inc.
    Written by Ulrich Drepper <drepper@gnu.ai.mit.edu>, April 1995.
 
    This program is free software: you can redistribute it and/or modify
@@ -34,6 +34,7 @@
 #include <sys/stat.h>
 #include <locale.h>
 #include <limits.h>
+#include <sys/resource.h>
 
 #if HAVE_ICONV
 #include <iconv.h>
@@ -52,6 +53,10 @@
 #include "dir-list.h"
 #include "file-list.h"
 #include "str-list.h"
+#include "gl_set.h"
+#include "gl_xset.h"
+#include "gl_hash_set.h"
+#include "hashkey-string.h"
 #include "error-progname.h"
 #include "progname.h"
 #include "relocatable.h"
@@ -80,7 +85,10 @@
 #include "msgl-ascii.h"
 #include "msgl-ofn.h"
 #include "xg-check.h"
+#include "stat-time.h"
+#include "vc-mtime.h"
 #include "po-time.h"
+#include "msgl-header.h"
 #include "write-catalog.h"
 #include "write-po.h"
 #include "write-properties.h"
@@ -105,14 +113,20 @@
 #include "x-java.h"
 #include "x-csharp.h"
 #include "x-javascript.h"
+#include "x-typescript.h"
+#include "x-typescriptx.h"
 #include "x-scheme.h"
 #include "x-lisp.h"
 #include "x-elisp.h"
 #include "x-librep.h"
+#include "x-rust.h"
+#include "x-go.h"
 #include "x-ruby.h"
 #include "x-sh.h"
 #include "x-awk.h"
 #include "x-lua.h"
+#include "x-modula2.h"
+#include "x-d.h"
 #include "x-smalltalk.h"
 #include "x-vala.h"
 #include "x-tcl.h"
@@ -176,6 +190,9 @@ static catalog_output_format_ty output_syntax = &output_format_po;
 /* If nonzero omit header with information about this run.  */
 int xgettext_omit_header;
 
+/* If nonzero, don't use 'git' to compute a reproducible POT-Creation-Date.  */
+static int xgettext_no_git;
+
 /* Be more verbose.  */
 int verbose = 0;
 
@@ -190,14 +207,23 @@ static flag_context_list_table_ty flag_table_python;
 static flag_context_list_table_ty flag_table_java;
 static flag_context_list_table_ty flag_table_csharp;
 static flag_context_list_table_ty flag_table_javascript;
+static flag_context_list_table_ty flag_table_typescript;
+static flag_context_list_table_ty flag_table_typescriptx;
 static flag_context_list_table_ty flag_table_scheme;
 static flag_context_list_table_ty flag_table_lisp;
 static flag_context_list_table_ty flag_table_elisp;
 static flag_context_list_table_ty flag_table_librep;
+#if 0 /* declared in x-rust.h */
+extern flag_context_list_table_ty flag_table_rust_functions;
+extern flag_context_list_table_ty flag_table_rust_macros;
+#endif
+static flag_context_list_table_ty flag_table_go;
 static flag_context_list_table_ty flag_table_ruby;
 static flag_context_list_table_ty flag_table_sh;
 static flag_context_list_table_ty flag_table_awk;
 static flag_context_list_table_ty flag_table_lua;
+static flag_context_list_table_ty flag_table_modula2;
+static flag_context_list_table_ty flag_table_d;
 static flag_context_list_table_ty flag_table_vala;
 static flag_context_list_table_ty flag_table_tcl;
 static flag_context_list_table_ty flag_table_perl;
@@ -227,6 +253,13 @@ static locating_rule_list_ty *its_locating_rules;
 /* If nonzero add comments used by itstool.  */
 static bool add_itstool_comments = false;
 
+/* The file names whose version-controlled modification times shall be
+   considered.  */
+static string_list_ty files_for_vc_mtime;
+/* The file names whose (possibly version-controlled) modification times
+   shall be ignored.  */
+static gl_set_t generated_files;
+
 /* Long options.  */
 static const struct option long_options[] =
 {
@@ -248,6 +281,7 @@ static const struct option long_options[] =
   { "force-po", no_argument, &force_po, 1 },
   { "foreign-user", no_argument, NULL, CHAR_MAX + 2 },
   { "from-code", required_argument, NULL, CHAR_MAX + 3 },
+  { "generated", required_argument, NULL, CHAR_MAX + 24 },
   { "help", no_argument, NULL, 'h' },
   { "indent", no_argument, NULL, 'i' },
   { "its", required_argument, NULL, CHAR_MAX + 20 },
@@ -260,6 +294,7 @@ static const struct option long_options[] =
   { "msgstr-prefix", optional_argument, NULL, 'm' },
   { "msgstr-suffix", optional_argument, NULL, 'M' },
   { "no-escape", no_argument, NULL, 'e' },
+  { "no-git", no_argument, NULL, CHAR_MAX + 23 },
   { "no-location", no_argument, NULL, CHAR_MAX + 16 },
   { "no-wrap", no_argument, NULL, CHAR_MAX + 4 },
   { "omit-header", no_argument, &xgettext_omit_header, 1 },
@@ -269,6 +304,7 @@ static const struct option long_options[] =
   { "package-version", required_argument, NULL, CHAR_MAX + 13 },
   { "properties-output", no_argument, NULL, CHAR_MAX + 6 },
   { "qt", no_argument, NULL, CHAR_MAX + 9 },
+  { "reference", required_argument, NULL, CHAR_MAX + 22 },
   { "sentence-end", required_argument, NULL, CHAR_MAX + 18 },
   { "sort-by-file", no_argument, NULL, 'F' },
   { "sort-output", no_argument, NULL, 's' },
@@ -359,6 +395,7 @@ main (int argc, char *argv[])
 
   /* Set the text message domain.  */
   bindtextdomain (PACKAGE, relocate (LOCALEDIR));
+  bindtextdomain ("gnulib", relocate (GNULIB_LOCALEDIR));
   bindtextdomain ("bison-runtime", relocate (BISON_LOCALEDIR));
   textdomain (PACKAGE);
 
@@ -367,6 +404,10 @@ main (int argc, char *argv[])
 
   /* Set initial value of variables.  */
   default_domain = MESSAGE_DOMAIN_DEFAULT;
+  string_list_init (&files_for_vc_mtime);
+  generated_files =
+    gl_set_create_empty (GL_HASH_SET,
+                         hashkey_string_equals, hashkey_string_hash, NULL);
   xgettext_global_source_encoding = NULL;
   init_flag_table_c ();
   init_flag_table_objc ();
@@ -375,14 +416,20 @@ main (int argc, char *argv[])
   init_flag_table_java ();
   init_flag_table_csharp ();
   init_flag_table_javascript ();
+  init_flag_table_typescript ();
+  init_flag_table_typescriptx ();
   init_flag_table_scheme ();
   init_flag_table_lisp ();
   init_flag_table_elisp ();
   init_flag_table_librep ();
+  init_flag_table_rust ();
+  init_flag_table_go ();
   init_flag_table_ruby ();
   init_flag_table_sh ();
   init_flag_table_awk ();
   init_flag_table_lua ();
+  init_flag_table_modula2 ();
+  init_flag_table_d ();
   init_flag_table_vala ();
   init_flag_table_tcl ();
   init_flag_table_perl ();
@@ -400,22 +447,28 @@ main (int argc, char *argv[])
 
       case 'a':
         x_c_extract_all ();
-        x_sh_extract_all ();
         x_python_extract_all ();
+        x_java_extract_all ();
+        x_csharp_extract_all ();
+        x_javascript_extract_all ();
+        x_typescript_extract_all ();
+        x_typescriptx_extract_all ();
+        x_scheme_extract_all ();
         x_lisp_extract_all ();
         x_elisp_extract_all ();
         x_librep_extract_all ();
-        x_scheme_extract_all ();
-        x_java_extract_all ();
-        x_csharp_extract_all ();
+        x_rust_extract_all ();
+        x_go_extract_all ();
+        x_ruby_extract_all ();
+        x_sh_extract_all ();
         x_awk_extract_all ();
+        x_lua_extract_all ();
+        x_modula2_extract_all ();
+        x_d_extract_all ();
+        x_vala_extract_all ();
         x_tcl_extract_all ();
         x_perl_extract_all ();
         x_php_extract_all ();
-        x_ruby_extract_all ();
-        x_lua_extract_all ();
-        x_javascript_extract_all ();
-        x_vala_extract_all ();
         break;
 
       case 'c':
@@ -480,22 +533,28 @@ main (int argc, char *argv[])
           optarg = NULL;
         x_c_keyword (optarg);
         x_objc_keyword (optarg);
-        x_sh_keyword (optarg);
         x_python_keyword (optarg);
+        x_java_keyword (optarg);
+        x_csharp_keyword (optarg);
+        x_javascript_keyword (optarg);
+        x_typescript_keyword (optarg);
+        x_typescriptx_keyword (optarg);
+        x_scheme_keyword (optarg);
         x_lisp_keyword (optarg);
         x_elisp_keyword (optarg);
         x_librep_keyword (optarg);
-        x_scheme_keyword (optarg);
-        x_java_keyword (optarg);
-        x_csharp_keyword (optarg);
+        x_rust_keyword (optarg);
+        x_go_keyword (optarg);
+        x_ruby_keyword (optarg);
+        x_sh_keyword (optarg);
         x_awk_keyword (optarg);
+        x_lua_keyword (optarg);
+        x_modula2_keyword (optarg);
+        x_d_keyword (optarg);
+        x_vala_keyword (optarg);
         x_tcl_keyword (optarg);
         x_perl_keyword (optarg);
         x_php_keyword (optarg);
-        x_ruby_keyword (optarg);
-        x_lua_keyword (optarg);
-        x_javascript_keyword (optarg);
-        x_vala_keyword (optarg);
         x_desktop_keyword (optarg);
         if (optarg == NULL)
           no_default_keywords = true;
@@ -685,6 +744,18 @@ main (int argc, char *argv[])
         x_javascript_tag (optarg);
         break;
 
+      case CHAR_MAX + 22: /* --reference */
+        string_list_append (&files_for_vc_mtime, optarg);
+        break;
+
+      case CHAR_MAX + 23: /* --no-git */
+        xgettext_no_git = true;
+        break;
+
+      case CHAR_MAX + 24: /* --generated */
+        gl_set_add (generated_files, optarg);
+        break;
+
       default:
         usage (EXIT_FAILURE);
         /* NOTREACHED */
@@ -701,7 +772,7 @@ License GPLv3+: GNU GPL version 3 or later <%s>\n\
 This is free software: you are free to change and redistribute it.\n\
 There is NO WARRANTY, to the extent permitted by law.\n\
 "),
-              "1995-2024", "https://gnu.org/licenses/gpl.html");
+              "1995-2025", "https://gnu.org/licenses/gpl.html");
       printf (_("Written by %s.\n"), proper_name ("Ulrich Drepper"));
       exit (EXIT_SUCCESS);
     }
@@ -799,7 +870,11 @@ xgettext cannot work without keywords to look for"));
 
   /* Determine list of files we have to process.  */
   if (files_from != NULL)
-    file_list = read_names_from_file (files_from);
+    {
+      if (strcmp (files_from, "-") != 0)
+        string_list_append (&files_for_vc_mtime, files_from);
+      file_list = read_names_from_file (files_from);
+    }
   else
     file_list = string_list_alloc ();
   /* Append names from command line.  */
@@ -1105,11 +1180,13 @@ If output file is -, output is written to standard output.\n"));
 Choice of input file language:\n"));
       printf (_("\
   -L, --language=NAME         recognise the specified language\n\
-                                (C, C++, ObjectiveC, PO, Shell, Python, Lisp,\n\
-                                EmacsLisp, librep, Scheme, Guile, Smalltalk,\n\
-                                Java, JavaProperties, C#, awk, YCP, Tcl, Perl,\n\
-                                PHP, Ruby, GCC-source, NXStringTable, RST, RSJ,\n\
-                                Glade, Lua, JavaScript, Vala, Desktop)\n"));
+                                (C, C++, ObjectiveC, PO, Python, Java,\n\
+                                JavaProperties, C#, JavaScript, TypeScript, TSX,\n\
+                                Scheme, Guile, Lisp, EmacsLisp, librep, Rust,\n\
+                                Go, Ruby, Shell, awk, Lua, Modula-2, D,\n\
+                                Smalltalk, Vala, Tcl, Perl, PHP, GCC-source,\n\
+                                YCP, NXStringTable, RST, RSJ, Glade, GSettings,\n\
+                                Desktop)\n"));
       printf (_("\
   -C, --c++                   shorthand for --language=C++\n"));
       printf (_("\
@@ -1148,26 +1225,29 @@ Language specific options:\n"));
       printf (_("\
   -a, --extract-all           extract all strings\n"));
       printf (_("\
-                                (only languages C, C++, ObjectiveC, Shell,\n\
-                                Python, Lisp, EmacsLisp, librep, Scheme, Java,\n\
-                                C#, awk, Tcl, Perl, PHP, GCC-source, Glade,\n\
-                                Lua, JavaScript, Vala)\n"));
+                                (only languages C, C++, ObjectiveC, Python,\n\
+                                Java, C#, JavaScript, TypeScript, TSX, Scheme,\n\
+                                Guile, Lisp, EmacsLisp, librep, Rust, Go, Shell,\n\
+                                awk, Lua, Modula-2, D, Vala, Tcl, Perl, PHP,\n\
+                                GCC-source, Glade, GSettings)\n"));
       printf (_("\
   -kWORD, --keyword=WORD      look for WORD as an additional keyword\n\
   -k, --keyword               do not to use default keywords\n"));
       printf (_("\
-                                (only languages C, C++, ObjectiveC, Shell,\n\
-                                Python, Lisp, EmacsLisp, librep, Scheme, Java,\n\
-                                C#, awk, Tcl, Perl, PHP, GCC-source, Glade,\n\
-                                Lua, JavaScript, Vala, Desktop)\n"));
+                                (only languages C, C++, ObjectiveC, Python,\n\
+                                Java, C#, JavaScript, TypeScript, TSX, Scheme,\n\
+                                Guile, Lisp, EmacsLisp, librep, Rust, Go, Shell,\n\
+                                awk, Lua, Modula-2, D, Vala, Tcl, Perl, PHP,\n\
+                                GCC-source, Glade, GSettings, Desktop)\n"));
       printf (_("\
       --flag=WORD:ARG:FLAG    additional flag for strings inside the argument\n\
                               number ARG of keyword WORD\n"));
       printf (_("\
-                                (only languages C, C++, ObjectiveC, Shell,\n\
-                                Python, Lisp, EmacsLisp, librep, Scheme, Java,\n\
-                                C#, awk, YCP, Tcl, Perl, PHP, GCC-source,\n\
-                                Lua, JavaScript, Vala)\n"));
+                                (only languages C, C++, ObjectiveC, Python,\n\
+                                Java, C#, JavaScript, TypeScript, TSX, Scheme,\n\
+                                Guile, Lisp, EmacsLisp, librep, Rust, Go, Shell,\n\
+                                awk, Lua, Modula-2, D, Vala, Tcl, Perl, PHP,\n\
+                                GCC-source, YCP)\n"));
       printf (_("\
       --tag=WORD:FORMAT       defines the behaviour of tagged template literals\n\
                               with tag WORD\n"));
@@ -1236,6 +1316,10 @@ Output details:\n"));
       printf (_("\
       --omit-header           don't write header with 'msgid \"\"' entry\n"));
       printf (_("\
+      --no-git                don't use the git program to produce a\n\
+                              reproducible 'POT-Creation-Date' field in the\n\
+                              output.\n"));
+      printf (_("\
       --copyright-holder=STRING  set copyright holder in output\n"));
       printf (_("\
       --foreign-user          omit FSF copyright in output for foreign user\n"));
@@ -1246,6 +1330,14 @@ Output details:\n"));
       printf (_("\
       --msgid-bugs-address=EMAIL@ADDRESS  set report address for msgid bugs\n"));
       printf (_("\
+      --generated=FILE        Declares that the given FILE is generated and\n\
+                              therefore should not have an influence on the\n\
+                              'POT-Creation-Date' field in the output.\n"));
+      printf (_("\
+      --reference=FILE        Declares that the output depends on the contents\n\
+                              of the given FILE.  This has an influence on the\n\
+                              'POT-Creation-Date' field in the output.\n"));
+      printf (_("\
   -m[STRING], --msgstr-prefix[=STRING]  use STRING or \"\" as prefix for msgstr\n\
                                 values\n"));
       printf (_("\
@@ -1348,9 +1440,11 @@ read_exclusion_file (char *filename)
 {
   char *real_filename;
   FILE *fp = open_catalog_file (filename, &real_filename, true);
-  abstract_catalog_reader_ty *catr;
 
-  catr = catalog_reader_alloc (&exclude_methods, textmode_xerror_handler);
+  string_list_append (&files_for_vc_mtime, real_filename);
+
+  abstract_catalog_reader_ty *catr =
+    catalog_reader_alloc (&exclude_methods, textmode_xerror_handler);
   catalog_reader_parse (catr, fp, real_filename, filename, true, &input_format_po);
   catalog_reader_free (catr);
 
@@ -1546,6 +1640,12 @@ xgettext_record_flag (const char *optionstring)
                                                         name_start, name_end,
                                                         argnum, value, pass);
                       }
+                    if (backend == NULL || strcmp (backend, "D") == 0)
+                      {
+                        flag_context_list_table_insert (&flag_table_d, XFORMAT_PRIMARY,
+                                                        name_start, name_end,
+                                                        argnum, value, pass);
+                      }
                     if (backend == NULL || strcmp (backend, "Vala") == 0)
                       {
                         flag_context_list_table_insert (&flag_table_vala, XFORMAT_PRIMARY,
@@ -1601,6 +1701,12 @@ xgettext_record_flag (const char *optionstring)
                     flag_context_list_table_insert (&flag_table_javascript, XFORMAT_PRIMARY,
                                                     name_start, name_end,
                                                     argnum, value, pass);
+                    flag_context_list_table_insert (&flag_table_typescript, XFORMAT_PRIMARY,
+                                                    name_start, name_end,
+                                                    argnum, value, pass);
+                    flag_context_list_table_insert (&flag_table_typescriptx, XFORMAT_PRIMARY,
+                                                    name_start, name_end,
+                                                    argnum, value, pass);
                     break;
                   case format_scheme:
                     flag_context_list_table_insert (&flag_table_scheme, XFORMAT_PRIMARY,
@@ -1622,6 +1728,21 @@ xgettext_record_flag (const char *optionstring)
                                                     name_start, name_end,
                                                     argnum, value, pass);
                     break;
+                  case format_rust:
+                    if (name_end - name_start > 1 && name_end[-1] == '!')
+                      flag_context_list_table_insert (&flag_table_rust_macros, XFORMAT_PRIMARY,
+                                                      name_start, name_end - 1,
+                                                      argnum, value, pass);
+                    else
+                      flag_context_list_table_insert (&flag_table_rust_functions, XFORMAT_PRIMARY,
+                                                      name_start, name_end,
+                                                      argnum, value, pass);
+                    break;
+                  case format_go:
+                    flag_context_list_table_insert (&flag_table_go, XFORMAT_PRIMARY,
+                                                    name_start, name_end,
+                                                    argnum, value, pass);
+                    break;
                   case format_ruby:
                     flag_context_list_table_insert (&flag_table_ruby, XFORMAT_PRIMARY,
                                                     name_start, name_end,
@@ -1644,6 +1765,16 @@ xgettext_record_flag (const char *optionstring)
                     break;
                   case format_pascal:
                     break;
+                  case format_modula2:
+                    flag_context_list_table_insert (&flag_table_modula2, XFORMAT_PRIMARY,
+                                                    name_start, name_end,
+                                                    argnum, value, pass);
+                    break;
+                  case format_d:
+                    flag_context_list_table_insert (&flag_table_d, XFORMAT_SECONDARY,
+                                                    name_start, name_end,
+                                                    argnum, value, pass);
+                    break;
                   case format_smalltalk:
                     break;
                   case format_qt:
@@ -1950,6 +2081,8 @@ extract_from_file (const char *file_name, extractor_ty extractor,
   if (extractor.extract_from_stream)
     {
       FILE *fp = xgettext_open (file_name, &logical_file_name, &real_file_name);
+      if (fp != stdin)
+        string_list_append (&files_for_vc_mtime, real_file_name);
 
       /* Set the default for the source file encoding.  May be overridden by
          the extractor function.  */
@@ -1971,6 +2104,7 @@ extract_from_file (const char *file_name, extractor_ty extractor,
       const char *found_in_dir;
       xgettext_find_file (file_name, &logical_file_name,
                           &found_in_dir, &real_file_name);
+      string_list_append (&files_for_vc_mtime, real_file_name);
 
       extractor.extract_from_file (found_in_dir, real_file_name,
                                    logical_file_name,
@@ -2022,6 +2156,8 @@ extract_from_xml_file (const char *file_name,
   char *logical_file_name;
   char *real_file_name;
   FILE *fp = xgettext_open (file_name, &logical_file_name, &real_file_name);
+  if (fp != stdin)
+    string_list_append (&files_for_vc_mtime, real_file_name);
 
   /* The default encoding for XML is UTF-8.  It can be overridden by
      an XML declaration in the XML file itself, not through the
@@ -2038,6 +2174,7 @@ extract_from_xml_file (const char *file_name,
 
   if (fp != stdin)
     fclose (fp);
+  string_list_append (&files_for_vc_mtime, real_file_name);
   free (logical_file_name);
   free (real_file_name);
 }
@@ -2055,8 +2192,6 @@ static message_ty *
 construct_header ()
 {
   char *project_id_version;
-  time_t now;
-  char *timestring;
   message_ty *mp;
   char *msgstr;
   char *comment;
@@ -2081,13 +2216,10 @@ the MSGID_BUGS_ADDRESS variable there; otherwise please\n\
 specify an --msgid-bugs-address command line option.\n\
 ")));
 
-  time (&now);
-  timestring = po_strftime (&now);
-
   msgstr = xasprintf ("\
 Project-Id-Version: %s\n\
 Report-Msgid-Bugs-To: %s\n\
-POT-Creation-Date: %s\n\
+POT-Creation-Date: \n\
 PO-Revision-Date: YEAR-MO-DA HO:MI+ZONE\n\
 Last-Translator: FULL NAME <EMAIL@ADDRESS>\n\
 Language-Team: LANGUAGE <LL@li.org>\n\
@@ -2096,10 +2228,8 @@ MIME-Version: 1.0\n\
 Content-Type: text/plain; charset=CHARSET\n\
 Content-Transfer-Encoding: 8bit\n",
                       project_id_version,
-                      msgid_bugs_address != NULL ? msgid_bugs_address : "",
-                      timestring);
+                      msgid_bugs_address != NULL ? msgid_bugs_address : "");
   assume (msgstr != NULL);
-  free (timestring);
   free (project_id_version);
 
   mp = message_alloc (NULL, "", NULL, msgstr, strlen (msgstr) + 1, &pos);
@@ -2125,9 +2255,143 @@ FIRST AUTHOR <EMAIL@ADDRESS>, YEAR.\n");
   return mp;
 }
 
+/* Accumulating mtimes.  */
+struct accumulator
+{
+  bool has_some_mtimes;
+  struct timespec max_of_mtimes;
+};
+
+static void
+accumulate (struct accumulator *accu, struct timespec mtime)
+{
+  if (accu->has_some_mtimes)
+    {
+      /* Compute the maximum of accu->max_of_mtimes and mtime.  */
+      if (accu->max_of_mtimes.tv_sec < mtime.tv_sec
+          || (accu->max_of_mtimes.tv_sec == mtime.tv_sec
+              && accu->max_of_mtimes.tv_nsec < mtime.tv_nsec))
+       accu->max_of_mtimes = mtime;
+    }
+  else
+    {
+      accu->max_of_mtimes = mtime;
+      accu->has_some_mtimes = true;
+    }
+}
+
+static int
+max_mtime_without_git (struct timespec *max_of_mtimes,
+                       size_t nfiles, const char * const *filenames)
+{
+  if (nfiles == 0)
+    /* Invalid argument.  */
+    abort ();
+
+  struct accumulator accu = { false };
+
+  /* Always use the file's time stamp.  */
+  for (size_t n = 0; n < nfiles; n++)
+    {
+      struct stat statbuf;
+      if (stat (filenames[n], &statbuf) < 0)
+        return -1;
+
+      struct timespec mtime = get_stat_mtime (&statbuf);
+      accumulate (&accu, mtime);
+    }
+
+  /* Since nfiles > 0, we must have accumulated at least one mtime.  */
+  if (!accu.has_some_mtimes)
+    abort ();
+  *max_of_mtimes = accu.max_of_mtimes;
+  return 0;
+}
+
+static int
+max_mtime (struct timespec *max_of_mtimes,
+           size_t nfiles, const char * const *filenames)
+{
+  if (xgettext_no_git)
+    return max_mtime_without_git (max_of_mtimes, nfiles, filenames);
+  else
+    {
+      /* Measure the time spent by the children processes (invocations of the
+         'git' program), and tell the user about the '--no-git' option if they
+         took more than 10 seconds.  */
+      struct rusage usage_before;
+      struct rusage usage_after;
+      bool usage_failed;
+      memset (&usage_before, '\0', sizeof (struct rusage));
+      usage_failed = getrusage (RUSAGE_CHILDREN, &usage_before) < 0;
+
+      int ret = max_vc_mtime (max_of_mtimes, nfiles, filenames);
+
+      memset (&usage_after, '\0', sizeof (struct rusage));
+      usage_failed |= getrusage (RUSAGE_CHILDREN, &usage_after) < 0;
+      if (!usage_failed)
+        {
+          long user_usec =
+            (usage_after.ru_utime.tv_sec - usage_before.ru_utime.tv_sec) * 1000000
+            + usage_after.ru_utime.tv_usec - usage_before.ru_utime.tv_usec;
+          long sys_usec =
+            (usage_after.ru_stime.tv_sec - usage_before.ru_stime.tv_sec) * 1000000
+            + usage_after.ru_stime.tv_usec - usage_before.ru_stime.tv_usec;
+          if (user_usec + sys_usec >= 10 * 1000000)
+            multiline_warning (xasprintf (_("warning: ")),
+                               xasprintf (_("\
+Determining the POT-Creation-Date through the\n\
+version-control (git) history took %.2f seconds.\n\
+If you want to speed this up and produce a less\n\
+reproducible POT-Creation-Date instead, without 'git',\n\
+pass the option '%s'.\n\
+"),
+                                          (user_usec + sys_usec) / 1000000.0,
+                                          "--no-git"));
+        }
+
+      return ret;
+    }
+}
+
 static void
 finalize_header (msgdomain_list_ty *mdlp)
 {
+  /* Set the POT-Creation-Date field.  */
+  {
+    /* First, filter out the generated files.  */
+    const char **filenames;
+    size_t nfiles;
+    {
+      const char **all_files = files_for_vc_mtime.item;
+      size_t num_all_files = files_for_vc_mtime.nitems;
+      filenames = XNMALLOC (num_all_files, const char *);
+      nfiles = 0;
+      for (size_t i = 0; i < num_all_files; i++)
+        {
+          const char *file = all_files[i];
+          if (!gl_set_search (generated_files, file))
+            filenames[nfiles++] = file;
+        }
+    }
+
+    /* Then, take the maximum of the (possibly version-controlled) modification
+       times of these files.  */
+    time_t stamp;
+    struct timespec max_of_mtimes;
+    if (nfiles > 0 && max_mtime (&max_of_mtimes, nfiles, filenames) == 0)
+      /* Use the maximum of the encountered mtimes.  */
+      stamp = max_of_mtimes.tv_sec;
+    else
+      /* Use the current time.  */
+      time (&stamp);
+    free (filenames);
+
+    char *timestring = po_strftime (&stamp);
+    msgdomain_list_set_header_field (mdlp, "POT-Creation-Date:", timestring);
+    free (timestring);
+  }
+
   /* If the generated PO file has plural forms, add a Plural-Forms template
      to the constructed header.  */
   {
@@ -2228,14 +2492,20 @@ language_to_extractor (const char *name)
     SCANNERS_JAVA
     SCANNERS_CSHARP
     SCANNERS_JAVASCRIPT
+    SCANNERS_TYPESCRIPT
+    SCANNERS_TYPESCRIPTX
     SCANNERS_SCHEME
     SCANNERS_LISP
     SCANNERS_ELISP
     SCANNERS_LIBREP
+    SCANNERS_RUST
+    SCANNERS_GO
     SCANNERS_RUBY
     SCANNERS_SH
     SCANNERS_AWK
     SCANNERS_LUA
+    SCANNERS_MODULA2
+    SCANNERS_D
     SCANNERS_SMALLTALK
     SCANNERS_VALA
     SCANNERS_TCL
@@ -2321,14 +2591,20 @@ extension_to_language (const char *extension)
     EXTENSIONS_JAVA
     EXTENSIONS_CSHARP
     EXTENSIONS_JAVASCRIPT
+    EXTENSIONS_TYPESCRIPT
+    EXTENSIONS_TYPESCRIPTX
     EXTENSIONS_SCHEME
     EXTENSIONS_LISP
     EXTENSIONS_ELISP
     EXTENSIONS_LIBREP
+    EXTENSIONS_RUST
+    EXTENSIONS_GO
     EXTENSIONS_RUBY
     EXTENSIONS_SH
     EXTENSIONS_AWK
     EXTENSIONS_LUA
+    EXTENSIONS_MODULA2
+    EXTENSIONS_D
     EXTENSIONS_SMALLTALK
     EXTENSIONS_VALA
     EXTENSIONS_TCL
